// =============================
//  class DATA opens a datafile (txt file with Mocap location data)
//  and reads the data into a class Data used in the Viewer
// 
//  created by M. Geroch
//             May-June 2001
// =====================================

import java.awt.*;
import java.io.*;
import java.util.*;
import java.applet.*;
import java.net.*;
import javax.vecmath.*;
import javax.media.j3d.*;
import java.lang.*;


public class Data {
  float[] knots, times;
  Point3f[][] positions;
  
  long secs;
  float mintime, maxtime;
  static boolean filled;
  int numframes, nummarkers;
  String setname;
  float minx, maxx, miny, maxy, minz, maxz;
  
  // ====================
  // constructors
  // ----------------------
  Data() {
    setname = null; 
    filled = false;
    mintime = maxtime = 0.0f;
    secs = 0;
    numframes = nummarkers = 0;
  }
  
  // ===========================
  // utilities
  // --------------------------
  public void clearData() {
    setname = null;
    filled = false;
    mintime = maxtime = 0.0f;
    secs = 0;
    numframes = nummarkers = 0;
  }
  
  public void setData(String movefile) {
    filled = false;
    try {
      getMovingInfo(movefile);
    }
    catch (Exception e) {
      System.out.println("Some early problem in datafile: "+movefile);
      System.out.println(e.toString());
      System.out.println(" -- aborting cleanly");
      clearData();
    }
    if (filled) {
	calcDerivedData();
    }
  }

  
  // ============================
  // getInfo -- opens file, reads in data, fills attributes
  // ---------------------------------
  public void getMovingInfo(String filename) throws Exception {
    try {
	  File input = new File(filename);
	  System.out.println(input);
	  FileReader in = new FileReader(input);
	  BufferedReader dis = new BufferedReader(in);
	  String data;
	  
      String sn = new String(filename);
      int slashindex = sn.lastIndexOf('/');
      int dotindex = sn.lastIndexOf('.');
      setname = sn.substring(slashindex+1, dotindex);
	  System.out.println("setname ok");
      
      // read in first line, count nummarkers
      data = dis.readLine();
	    
      StringTokenizer st = new StringTokenizer(data);
      if (st.nextToken().equals("PREPROCESSED")) {
	numframes = Integer.parseInt(st.nextToken());
	nummarkers = Integer.parseInt(st.nextToken());
	minx = Float.parseFloat(st.nextToken());
	maxx = Float.parseFloat(st.nextToken());
	miny = Float.parseFloat(st.nextToken());
	maxy = Float.parseFloat(st.nextToken());
	minz = Float.parseFloat(st.nextToken());
	maxz = Float.parseFloat(st.nextToken());

	// allocate size of arrays
	positions = new Point3f[numframes][nummarkers];
	knots = new float[numframes];
	times = new float[numframes];

	// read and extract data directly, no division by 1000.0f needed
	float x, y, z;
	int fr;
	for (int frnum= 0; frnum < numframes; ++frnum) {
	  data = dis.readLine();
	  st = new StringTokenizer(data);
	  fr = Integer.parseInt(st.nextToken());  // discard frame number
	  times[frnum] = Float.parseFloat(st.nextToken());
	  for (int mknum = 0; mknum < nummarkers; ++mknum) {
	    x = Float.parseFloat(st.nextToken()); 
	    y = Float.parseFloat(st.nextToken()); 
	    z = Float.parseFloat(st.nextToken()); 
	    positions[frnum][mknum] = new Point3f(x,y,z);
	  }
	}
	dis.close();
      }

      else {
	nummarkers = (st.countTokens()-2)/3;
	// need to get mins and maxes "manually"
	numframes = 0;
	
	// continue to read in lines, counting numframes
	String[] savedata = new String[3000];
	savedata[numframes] = dis.readLine();
	while (savedata[numframes] != null) {
	  numframes++;
	  savedata[numframes] = dis.readLine();
	}
	
	// close the file
	dis.close();
	System.out.println("read done");
	
	// allocate size of arrays
	positions = new Point3f[numframes][nummarkers];
	knots = new float[numframes];
	times = new float[numframes];
	
	// now parse the saved data
	float x, y, z;
	int fr;
	for (int frnum= 0; frnum < numframes; ++frnum) {
	  st = new StringTokenizer(savedata[frnum]);
	  fr = Integer.parseInt(st.nextToken());  // discard frame number
	  times[frnum] = Float.parseFloat(st.nextToken());
	  for (int mknum = 0; mknum < nummarkers; ++mknum) {
	    x = (Float.parseFloat(st.nextToken()))*0.025f;///1000.0f; 
	    y = (Float.parseFloat(st.nextToken()))*0.025f;///1000.0f; 
	    z = (Float.parseFloat(st.nextToken()))*0.025f;///1000.0f; 
	    positions[frnum][mknum] = new Point3f(x,y,z);
	  }
	}
	findRanges();
      }
      filled = true;
    }
    catch (Exception e) {
      System.out.println("Some problem in data file -- aborting");
      System.out.println(e.toString());
      clearData();
    }
  }

  // ===========================
  // findRanges -- finds mins and maxes if data wasn't preprocessed
  // ------------------------------
  private void findRanges() {
    // find the mins and maxes
    float x, y, z;
    minx = maxx = positions[0][0].x;
    miny = maxy = positions[0][0].y;
    minz = maxz = positions[0][0].z;
    for(int frnum = 0; frnum < numframes; ++frnum) {
      for (int mknum = 0; mknum < nummarkers; ++mknum) {
	x = positions[frnum][mknum].x;
	y = positions[frnum][mknum].y;
	z = positions[frnum][mknum].z;
	if (x <minx && x > -9.99)
	  minx = x;
	else if (x > maxx && x < 9.99)
	  maxx = x;
	if (y <miny && y > -9.99)
	  miny = y;
	else if (y > maxy && y < 9.99)
	  maxy = y;
	if (z <minz && z > -9.99)
	  minz = z;
	else if (z > maxz && z < 9.99)
	  maxz = z;
      }
    }
  }

  
  // ======================
  // calcDerivedData -- creates the appropriate derived data values
  // -----------------------
  public void calcDerivedData() {
    // calculate the secs and knots
    mintime = times[0];
    maxtime = times[numframes-1];
    knots[0] = 0.0f;
    secs = (long)(maxtime*1000);
    for (int j = 1; j < numframes; ++j) {
      knots[j] = times[j]/maxtime;
    }
  }

    // ===========================
    // printFileVals -- prints the values found in the file
    // ---------------------------
    public void printFileVals() {
	if (filled) {
	    System.out.println("\nFile values");
	    System.out.println("filename is "+setname);
	    System.out.println(numframes+" frames");
	    System.out.println(nummarkers+" markers");
	    for(int frnum = numframes-1; frnum < numframes; ++frnum) {
		System.out.println("fr "+frnum);
		  for (int mknum = nummarkers-3; mknum < nummarkers; ++mknum) 
		    System.out.println("mk "+mknum+" "+positions[frnum][mknum]);
	    }
	}
	else
	    System.out.println("Data is empty");
    }

    // =============================
    // printDerivedVals -- prints out the values that were derived from the data
    // ---------------------------------
    public void printDerivedVals() {
	if (filled) {
	    System.out.println("\nDerived Values");
	    System.out.println("filename is "+setname);
	    System.out.println(numframes+" frames");
	    System.out.println("Times from "+mintime+" to "+maxtime);
	    System.out.println("Creating "+secs+" secs for use in the alpha");
	    System.out.println("knots ...");
	    for (int frnum = numframes-1; frnum < numframes; ++frnum) {
		System.out.println(frnum+" : "+knots[frnum]);
	    }
	    System.out.println("x range is "+minx+" to "+maxx);
	    System.out.println("y range is "+miny+" to "+maxy);
	    System.out.println("z range is "+minz+" to "+maxz);
	}
    }

    // ======================
    // main - for testing Data
    // -----------------------
    public static void main(String args[]) throws Exception {
	Data d = new Data();
	String movingfile = "Data/me.txt";
	try {
	    d.setData(movingfile);
	}
	catch (Exception e) {
	    System.out.println("bad file reference .. aborting");
	    System.out.println(e);
	    System.exit(0);
	}
	if (filled) {
	    System.out.println("done filling arrays");
	    d.printFileVals();
	    d.printDerivedVals();
	}
	d.clearData();
	try {
	  d.setData("Data/vtnormJCP.txt");
	}
	catch (Exception e) {
	  System.out.println("second file problem");
	  System.out.println(e);
	  System.exit(0);
	}
	if (filled) {
	    System.out.println("done filling arrays");
	    d.printFileVals();
	    d.printDerivedVals();
	}
	
    }
}




